# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-03-02 15:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0016_purchaseslinealbaran_product_unique'),
        ('codenerix_storages', '0018_auto_20180227_0858'),
    ]

    operations = [
        migrations.CreateModel(
            name='Distribution',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('purchasesorder', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='distributions', to='codenerix_invoicing.PurchasesOrder', verbose_name='Order')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
        migrations.CreateModel(
            name='DistributionLine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('quantity', models.FloatField(default=1.0, verbose_name='Quantity')),
                ('expected_date', models.DateTimeField(blank=True, null=True, verbose_name='Desired Date')),
                ('distribution', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='distribution_lines', to='codenerix_storages.Distribution', verbose_name='Distribution List')),
                ('storage', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='distribution_lines', to='codenerix_storages.Storage', verbose_name='Storage')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-storages
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.db import models
from django.db.models import Q
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from codenerix.models import CodenerixModel
from codenerix_products.models import ProductFinal, ProductUnique, PRODUCT_UNIQUE_VALUE_LENGTH
from codenerix_invoicing.models_purchases import Provider, PurchasesOrder, PurchasesLineAlbaran
from codenerix_invoicing.models_sales import SalesAlbaran

from .models import Storage, StorageBox, StorageOperator, StorageZone


# Generic classes for Inventory and Inventory Line
class GenInventory(CodenerixModel):  # META: Abstract class
    end = models.DateTimeField(_("Ends"), blank=True, null=True, editable=False)
    notes = models.TextField(_("Notes"), blank=True, null=True)
    processed = models.BooleanField(_("Processed"), blank=False, null=False, default=False, editable=False)

    def lock_update(self):
        if self.processed:
            return _("This Inventory has been already processed, can not be updated")
        else:
            return None

    def lock_delete(self):
        if self.processed:
            return _("This Inventory has been already processed, can not be deleted")
        else:
            return None

    class Meta(CodenerixModel.Meta):
        abstract = True


class GenInventoryLine(CodenerixModel):  # META: Abstract class
    box = models.ForeignKey(StorageBox, on_delete=models.CASCADE, related_name='storage_%(class)s', verbose_name=_("Box"), null=False, blank=False)
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, related_name='storage_%(class)s', null=False, blank=False, verbose_name=_("Product Final"))
    product_unique = models.ForeignKey(ProductUnique, on_delete=models.CASCADE, related_name='storage_%(class)s', null=True, blank=True, verbose_name=_("Product Unique"))
    product_unique_value = models.CharField(_("Product Unique Value"), max_length=PRODUCT_UNIQUE_VALUE_LENGTH, blank=True, null=True, default=None, editable=False)
    operator = models.ForeignKey(StorageOperator, on_delete=models.CASCADE, related_name='storage_%(class)s', verbose_name=_("Storage Operator"), null=False, blank=False)
    quantity = models.FloatField(_("Quantity"), null=False, blank=False, default=1.0)
    notes = models.TextField(_("Notes"), blank=True, null=True)

    def __str__(self):
        return "{}::{}-{}-{}-{}".format(self.box, self.product_final, self.product_unique, self.product_unique_value, self.quantity)

    def lock_update(self):
        inventory = getattr(self, 'inventory', None)
        if inventory and inventory.processed:
            return _("This Inventory has been already processed, can not be updated")
        else:
            return None

    def lock_delete(self):
        inventory = getattr(self, 'inventory', None)
        if inventory and inventory.processed:
            return _("This Inventory has been already processed, can not be deleted")
        else:
            return None

    class Meta(CodenerixModel.Meta):
        abstract = True


# Inventory
class Inventory(GenInventory):
    storage = models.ForeignKey(Storage, on_delete=models.CASCADE, related_name='inventorys', verbose_name=_("Storage"), null=False, blank=False)
    zone = models.ForeignKey(StorageZone, on_delete=models.CASCADE, related_name='inventorys', verbose_name=_("Zone"), null=True, blank=True)

    def __fields__(self, info):
        fields = []
        fields.append(('storage', _('Storage')))
        fields.append(('zone', _('Zone')))
        fields.append(('created', _('Starts')))
        fields.append(('end', _('Ends')))
        fields.append(('notes', _('Notes')))
        fields.append(('processed', None))
        fields.append((None, _('Actions')))
        return fields

    def __str__(self):
        if self.zone:
            return u"{} - {}:{}".format(self.created, self.storage, self.zone)
        else:
            return u"{} - {}".format(self.created, self.storage)


class InventoryLine(GenInventoryLine):
    inventory = models.ForeignKey(Inventory, on_delete=models.CASCADE, related_name='storage_inventorys', verbose_name=_("Inventory"), null=False, blank=False)
    caducity = models.DateField(_("Caducity"), blank=True, null=True, default=None)

    def __fields__(self, info):
        fields = []
        fields.append(('box', _("Box")))
        fields.append(('product_final', _("Product")))
        fields.append(('product_unique', _("Unique")))
        fields.append(('product_unique_value', _("Unique Value")))
        fields.append(('operator', _("Operator")))
        fields.append(('quantity', _("Quantity")))
        fields.append(('caducity', _("Caducity")))
        fields.append(('notes', _("Notes")))
        return fields

    def __limitQ__(self, info):
        limit = {}
        ipk = info.kwargs.get('ipk', None)
        limit['file_link'] = Q(inventory__pk=ipk)
        return limit


# Distribution
class Distribution(CodenerixModel):
    purchasesorder = models.ForeignKey(PurchasesOrder, on_delete=models.CASCADE, related_name='distributions', verbose_name=_("Order"), null=True, blank=True)


class DistributionLine(CodenerixModel):
    distribution = models.ForeignKey(Distribution, on_delete=models.PROTECT, related_name='distribution_lines', verbose_name='Distribution List', blank=False, null=False)
    storage = models.ForeignKey(Storage, related_name='distribution_lines', verbose_name=_("Storage"), null=False, blank=False, on_delete=models.PROTECT)
    quantity = models.FloatField(_("Quantity"), null=False, blank=False, default=1.0)
    expected_date = models.DateTimeField(_("Desired Date"), blank=True, null=True)


# InventoryIn
class InventoryIn(GenInventory):
    provider = models.ForeignKey(Provider, on_delete=models.PROTECT, related_name='inventorys', verbose_name=_('Provider'), blank=False, null=False)

    def __fields__(self, info):
        fields = []
        fields.append(('provider', _('Provider')))
        fields.append(('created', _('Starts')))
        fields.append(('end', _('Ends')))
        fields.append(('notes', _('Notes')))
        fields.append((None, _('Actions')))
        return fields

    def __str__(self):
        return u"{}::{}".format(self.provider, self.created)


class InventoryInLine(GenInventoryLine):
    purchasesorder = models.ForeignKey(PurchasesOrder, on_delete=models.CASCADE, related_name='inventory_lines', verbose_name=_("Order"), null=True, blank=True)
    purchaseslinealbaran = models.ManyToManyField(PurchasesLineAlbaran, related_name='inventory_lines', verbose_name=_("Line Albaran"))
    inventory = models.ForeignKey(InventoryIn, on_delete=models.CASCADE, related_name='inventory_lines', verbose_name=_("Inventory"), null=False, blank=False)
    caducity = models.DateField(_("Caducity"), blank=True, null=True, default=None)

    def __fields__(self, info):
        fields = []
        fields.append(('purchasesorder', _("Purchases Order")))
        fields.append(('purchaseslinealbaran', _("Purchases Line Albaran")))
        fields.append(('box', _("Box")))
        fields.append(('product_final', _("Product")))
        fields.append(('product_unique', _("Unique")))
        fields.append(('product_unique_value', _("Unique Value")))
        fields.append(('operator', _("Operator")))
        fields.append(('quantity', _("Quantity")))
        fields.append(('caducity', _("Caducity")))
        fields.append(('notes', _("Notes")))
        return fields

    def __limitQ__(self, info):
        limit = {}
        ipk = info.kwargs.get('ipk', None)
        limit['file_link'] = Q(inventory__pk=ipk)
        return limit


# InventoryOut
class InventoryOut(GenInventory):
    albaran = models.ForeignKey(SalesAlbaran, on_delete=models.PROTECT, related_name='inventorys', verbose_name=_('Albaran'), blank=False, null=False)

    def __fields__(self, info):
        fields = []
        fields.append(('albaran', _('Albaran')))
        fields.append(('created', _('Starts')))
        fields.append(('end', _('Ends')))
        fields.append(('notes', _('Notes')))
        fields.append((None, _('Actions')))
        return fields

    def __str__(self):
        return u"{}::{}".format(self.albaran, self.created)


class InventoryOutLine(GenInventoryLine):
    inventory = models.ForeignKey(InventoryOut, on_delete=models.CASCADE, related_name='inventory_lines', verbose_name=_("Inventory line"), null=False, blank=False)

    def __fields__(self, info):
        fields = []
        fields.append(('box', _("Box")))
        fields.append(('product_final', _("Product")))
        fields.append(('product_unique', _("Unique")))
        fields.append(('product_unique_value', _("Unique Value")))
        fields.append(('operator', _("Operator")))
        fields.append(('quantity', _("Quantity")))
        fields.append(('notes', _("Notes")))
        return fields

    def __limitQ__(self, info):
        limit = {}
        ipk = info.kwargs.get('ipk', None)
        limit['file_link'] = Q(inventory__pk=ipk)
        return limit


# Solicitud de stock entre de almacenes
class RequestStock(CodenerixModel):
    storage_source = models.ForeignKey(Storage, related_name='request_stocks_src', verbose_name=_("Storage source"), null=False, blank=False, on_delete=models.PROTECT)
    storage_destination = models.ForeignKey(Storage, related_name='request_stocks_dst', verbose_name=_("Storage destionation"), null=False, blank=False, on_delete=models.PROTECT)
    request_user = models.ForeignKey(User, on_delete=models.CASCADE, blank=False, null=False, related_name='request_stocks')
    request_date = models.DateTimeField(_("Request Date"), blank=False, null=False)
    desired_date = models.DateTimeField(_("Desired Date"), blank=False, null=False)

    def __fields__(self, info):
        fields = []
        fields.append(('batch_source', _('Batch source'), 100))
        fields.append(('batch_destination', _('Batch destination'), 100))
        fields.append(('stock_movement_products', _('Products'), 100))
        fields.append(('storage_source', _("Storage source")))
        fields.append(('storage_destination', _("Storage destionation")))
        fields.append(('request_user', _('Request user')))
        fields.append(('request_date', _("Request Date")))
        fields.append(('desired_date', _("Desired Date")))
        return fields

    def __str__(self):
        return u"{} -> {}".format(self.storage_source, self.storage_destination)

    def lock_delete(self):
        if self.line_request_stock.exists():
            return _("Cannot delete request stock model, relationship between request stock model and lines")
        if self.outgoing_albarans.exists():
            return _("Cannot delete request stock model, relationship between request stock model and outgoing albaran")
        else:
            return super(RequestStock, self).lock_delete()

    def save(self, *args, **kwargs):
        self.code_format = getattr(settings, 'CDNX_STORAGE_CODE_REQUEST_STOCK', 'RS{year}{day}{month}-{hour}{minute}--{number}')
        return super(RequestStock, self).save(*args, **kwargs)


class LineRequestStock(CodenerixModel):
    request_stock = models.ForeignKey(RequestStock, on_delete=models.CASCADE, related_name='line_request_stock', verbose_name=_("Request stock"), null=False, blank=False)
    product_final = models.ForeignKey(ProductFinal, on_delete=models.CASCADE, related_name='line_request_stock', verbose_name=_("Product"), null=False, blank=False)
    quantity = models.FloatField(_("Quantity"), null=False, blank=False)

    def __fields__(self, info):
        fields = []
        fields.append(('product_final', _('Product'), 100))
        fields.append(('quantity', _('Quantity'), 100))
        return fields

    def __str__(self):
        return u"{} ({})".format(self.product_final, self.quantity)


class OutgoingAlbaran(CodenerixModel):
    inventory = models.ForeignKey(InventoryOut, on_delete=models.PROTECT, related_name='outgoing_albarans', verbose_name=_('Inventory'), blank=True, null=True)
    request_stock = models.ForeignKey(RequestStock, on_delete=models.CASCADE, related_name='outgoing_albarans', verbose_name=_("Request stock"), null=True, blank=True)
    prepare_user = models.ForeignKey(User, on_delete=models.CASCADE, blank=False, null=False, related_name='outgoing_albarans')
    prepare_date = models.DateTimeField(_("Prepare Date"), blank=False, null=False)
    outgoing_date = models.DateTimeField(_("Outgoing Date"), blank=True, null=True)
    estimated_date = models.DateTimeField(_("Estimated Date"), blank=True, null=True)

    def __fields__(self, info):
        fields = []
        fields.append(('request_stock', _("Request stock")))
        fields.append(('prepare_user', _('Prepare user')))
        fields.append(('prepare_date', _("Prepare Date")))
        fields.append(('outgoing_date', _("Outgoing Date")))
        fields.append(('estimated_date', _("Estimated Date")))
        return fields

    def __str__(self):
        return u"{} ({})".format(self.outgoing_date, self.request_stock)

    def lock_delete(self):
        if self.line_outgoing_albarans.exists():
            return _("Cannot delete outgoing albaran model, relationship between outgoing albaran model and lines")
        if self.incoming_albarans.exists():
            return _("Cannot delete outgoing albaran model, relationship between outgoing albaran model and incoming albaran")
        else:
            return super(OutgoingAlbaran, self).lock_delete()

    def save(self, *args, **kwargs):
        self.code_format = getattr(settings, 'CDNX_STORAGE_CODE_OUTGOING_ALBARAN', 'OA{year}{day}{month}-{hour}{minute}--{number}')
        return super(OutgoingAlbaran, self).save(*args, **kwargs)


class LineOutgoingAlbaran(CodenerixModel):
    outgoing_albaran = models.ForeignKey(OutgoingAlbaran, related_name='line_outgoing_albarans', verbose_name=_("Outgoing Albaran"), null=False, blank=False, on_delete=models.CASCADE)
    product_unique = models.ForeignKey(ProductUnique, related_name='line_outgoing_albarans', verbose_name=_("Product Unique"), null=False, blank=False, on_delete=models.CASCADE)
    # prepare_user = models.ForeignKey(User, related_name='line_outgoing_albarans_prepare', verbose_name=_("Prepare user"), blank=False, null=False, on_delete=models.CASCADE)
    validator_user = models.ForeignKey(User, related_name='line_outgoing_albarans_validator', verbose_name=_("Validator user"), blank=False, null=False, on_delete=models.CASCADE)
    # box = 1

    def __fields__(self, info):
        fields = []
        fields.append(('outgoing_albaran', _("Outgoing Albaran")))
        fields.append(('product_unique', _("Product Unique")))
        fields.append(('prepare_user', _("Prepare user")))
        fields.append(('validator_user', _("Validator user")))
        # box = 1
        return fields

    def __str__(self):
        return u"{} ({})".format(self.outgoing_albaran, self.product_unique)


class IncomingAlbaran(CodenerixModel):
    outgoing_albaran = models.ForeignKey(OutgoingAlbaran, related_name='incoming_albarans', verbose_name=_("Outgoing Albaran"), null=False, blank=False, on_delete=models.CASCADE)
    reception_user = models.ForeignKey(User, related_name='incoming_albarans', verbose_name=_("Reception user"), blank=False, null=False, on_delete=models.CASCADE)
    reception_date = models.DateTimeField(_("Reception Date"), blank=True, null=True)

    def __fields__(self, info):
        fields = []
        fields.append(('outgoing_albaran', _("Outgoing Albaran")))
        fields.append(('reception_user', _("Reception user")))
        fields.append(('reception_date', _("Reception Date")))
        return fields

    def __str__(self):
        return u"{} ({})".format(self.outgoing_albaran, self.reception_user)

    def lock_delete(self):
        if self.line_incoming_albarans.exists():
            return _("Cannot delete incoming albaran model, relationship between incoming albaran model and lines")
        else:
            return super(IncomingAlbaran, self).lock_delete()

    def save(self, *args, **kwargs):
        self.code_format = getattr(settings, 'CDNX_STORAGE_CODE_INGOING_ALBARAN', 'OI{year}{day}{month}-{hour}{minute}--{number}')
        return super(IncomingAlbaran, self).save(*args, **kwargs)


class LineIncomingAlbaran(CodenerixModel):
    incoming_albaran = models.ForeignKey(IncomingAlbaran, related_name='line_incoming_albarans', verbose_name=_("Incoming Albaran"), null=False, blank=False, on_delete=models.CASCADE)
    box = models.ForeignKey(StorageBox, related_name='line_ingoming_albarans', verbose_name=_("Box"), null=False, blank=False, on_delete=models.CASCADE)
    product_unique = models.ForeignKey(OutgoingAlbaran, related_name='line_incoming_albarans', verbose_name=_("Product"), null=False, blank=False, on_delete=models.CASCADE)
    quantity = models.FloatField(_("Quantity"), null=False, blank=False)
    validator_user = models.ForeignKey(User, related_name='line_incoming_albarans', verbose_name=_("Validator user"), blank=False, null=False, on_delete=models.CASCADE)

    def __fields__(self, info):
        fields = []
        fields.append(('incoming_albaran', _("Incoming Albaran")))
        fields.append(('box', _("Box")))
        fields.append(('product_unique', _("Product")))
        fields.append(('quantity', _("Quantity")))
        fields.append(('validator_user', _("Validator user")))
        return fields

    def __str__(self):
        return u"{} ({})".format(self.product_unique, self.quantity)



# -*- coding: utf-8 -*-
#
# django-codenerix-storages
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib

from django.db.models import Q
from django.db.models.fields import FieldDoesNotExist
from django.forms.utils import ErrorList
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _
from django.conf import settings

from codenerix_extensions.views import GenCreateBridge, GenUpdateBridge
from codenerix.views import GenList, GenCreate, GenCreateModal, GenUpdate, GenUpdateModal, GenDelete, GenDetail, GenDetailModal, GenForeignKey

from codenerix_storages.models import Storage, StorageZone
from codenerix_storages.models import StorageBox, StorageOperator
from codenerix_storages.models import StorageBoxStructure, StorageBoxKind

from codenerix_storages.forms import StorageForm
from codenerix_storages.forms import StorageZoneForm, StorageZoneOwnForm
from codenerix_storages.forms import StorageBoxForm, StorageBoxFormUpdate, StorageOperatorForm
from codenerix_storages.forms import StorageBoxStructureForm, StorageBoxKindForm


# ###########################################
class GenStorageUrl(object):
    ws_entry_point = '{}/storages'.format(settings.CDNX_STORAGES_URL_COMMON)


# Storage
class StorageList(GenStorageUrl, GenList):
    model = Storage
    show_details = True
    extra_context = {'menu': ['storage', 'storage'], 'bread': [_('Storage'), _('Storage')]}


class StorageCreate(GenStorageUrl, GenCreate, GenCreateBridge):
    model = Storage
    form_class = StorageForm


class StorageCreateModal(GenCreateModal, StorageCreate):
    pass


class StorageUpdate(GenStorageUrl, GenUpdate, GenUpdateBridge):
    model = Storage
    show_details = True
    form_class = StorageForm


class StorageUpdateModal(GenUpdateModal, StorageUpdate):
    pass


class StorageDelete(GenStorageUrl, GenDelete):
    model = Storage


class StorageDetails(GenStorageUrl, GenDetail):
    model = Storage
    groups = StorageForm.__groups_details__()
    tabs = [
        {'id': 'Zone', 'name': _('Zones'), 'ws': 'CDNX_storages_storagezones_sublist', 'rows': 'base'},
    ]


class StorageSubList(GenStorageUrl, GenList):
    model = Storage
    show_details = False
    json = False
    extra_context = {'menu': ['Storage', 'storage'], 'bread': [_('Storage'), _('Storage')]}

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['file_link'] = Q(storage_zones__pk=pk)
        return limit


# ###########################################
class GenStorageZoneUrl(object):
    ws_entry_point = '{}/storagezones'.format(settings.CDNX_STORAGES_URL_COMMON)


# StorageZone
class StorageZoneList(GenStorageZoneUrl, GenList):
    model = StorageZone
    show_details = True
    extra_context = {'menu': ['StorageZone', 'storage'], 'bread': [_('StorageZone'), _('Storage')]}


class StorageZoneCreate(GenStorageZoneUrl, GenCreate):
    model = StorageZone
    form_class = StorageZoneForm


class StorageZoneCreateModal(GenCreateModal, StorageZoneCreate):
    pass


class OwnStorageZoneCreateModal(GenCreateModal, StorageZoneCreate):
    form_class = StorageZoneOwnForm

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        self.__storage_pk = kwargs.get('pk', None)
        return super(StorageZoneCreate, self).dispatch(*args, **kwargs)

    def form_valid(self, form):
        if self.__storage_pk:
            storage = Storage.objects.get(pk=self.__storage_pk)
            self.request.storage = storage
            form.instance.storage = storage

        return super(StorageZoneCreate, self).form_valid(form)


class StorageZoneUpdate(GenStorageZoneUrl, GenUpdate):
    model = StorageZone
    show_details = True
    form_class = StorageZoneForm


class StorageZoneUpdateModal(GenUpdateModal, StorageZoneUpdate):
    pass


class OwnStorageZoneUpdateModal(GenUpdateModal, StorageZoneUpdate):
    form_class = StorageZoneOwnForm


class StorageZoneDelete(GenStorageZoneUrl, GenDelete):
    model = StorageZone


class StorageZoneSubList(GenStorageZoneUrl, GenList):
    model = StorageZone
    field_delete = True

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['file_link'] = Q(storage__pk=pk)
        return limit


class StorageZoneDetail(GenStorageZoneUrl, GenDetail):
    model = StorageZone
    groups = StorageZoneForm.__groups_details__()
    tabs = []
    exclude_fields = []


class StorageZoneDetailModal(GenDetailModal, StorageZoneDetail):
    pass


# ###########################################
class GenStorageBoxUrl(object):
    ws_entry_point = '{}/storageboxs'.format(settings.CDNX_STORAGES_URL_COMMON)


# StorageBox
class StorageBoxList(GenStorageBoxUrl, GenList):
    model = StorageBox
    show_details = True
    extra_context = {'menu': ['StorageBox', 'storage'], 'bread': [_('StorageBox'), _('Storage')]}


class StorageBoxCreate(GenStorageBoxUrl, GenCreate):
    model = StorageBox
    form_class = StorageBoxForm


class StorageBoxCreateModal(GenCreateModal, StorageBoxCreate):
    pass


class StorageBoxUpdate(GenStorageBoxUrl, GenUpdate):
    model = StorageBox
    show_details = True
    form_class = StorageBoxFormUpdate


class StorageBoxUpdateModal(GenUpdateModal, StorageBoxUpdate):
    pass


class StorageBoxDelete(GenStorageBoxUrl, GenDelete):
    model = StorageBox


class StorageBoxSubList(GenStorageBoxUrl, GenList):
    model = StorageBox
    field_delete = True

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['file_link'] = Q(storage__pk=pk)
        return limit


class StorageBoxDetail(GenStorageBoxUrl, GenDetail):
    model = StorageBox
    groups = StorageBoxForm.__groups_details__()
    tabs = []
    exclude_fields = []


class StorageBoxDetailModal(GenDetailModal, StorageBoxDetail):
    pass


class StorageBoxForeign(GenForeignKey):
    model = StorageBox
#    label = "{name}"
    label = '{<LANGUAGE_CODE>__name}'

    def get_foreign(self, queryset, search, filter):
        return queryset.all()


# ###########################################
class StorageBoxStructureUrl(object):
    ws_entry_point = '{}/storageboxstructures'.format(settings.CDNX_STORAGES_URL_COMMON)


# StorageBoxStructure
class StorageBoxStructureList(StorageBoxStructureUrl, GenList):
    model = StorageBoxStructure
    extra_context = {'menu': ['StorageBoxStructure', 'storage'], 'bread': [_('StorageBoxStructure'), _('Storage')]}


class StorageBoxStructureCreate(StorageBoxStructureUrl, GenCreate):
    model = StorageBoxStructure
    form_class = StorageBoxStructureForm


class StorageBoxStructureCreateModal(GenCreateModal, StorageBoxStructureCreate):
    pass


class StorageBoxStructureUpdate(StorageBoxStructureUrl, GenUpdate):
    model = StorageBoxStructure
    form_class = StorageBoxStructureForm


class StorageBoxStructureUpdateModal(GenUpdateModal, StorageBoxStructureUpdate):
    pass


class StorageBoxStructureDelete(StorageBoxStructureUrl, GenDelete):
    model = StorageBoxStructure


class StorageBoxStructureSubList(StorageBoxStructureUrl, GenList):
    model = StorageBoxStructure
    show_details = False
    extra_context = {'menu': ['StorageBoxStructure', 'storage'], 'bread': [_('StorageBoxStructure'), _('Storage')]}

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(xxxxxxx__pk=pk)
        return limit


class StorageBoxStructureDetails(StorageBoxStructureUrl, GenDetail):
    model = StorageBoxStructure
    groups = StorageBoxStructureForm.__groups_details__()


class StorageBoxStructureDetailModal(GenDetailModal, StorageBoxStructureDetails):
    pass


# ###########################################
class StorageBoxKindUrl(object):
    ws_entry_point = '{}/storageboxkinds'.format(settings.CDNX_STORAGES_URL_COMMON)


# StorageBoxKind
class StorageBoxKindList(StorageBoxKindUrl, GenList):
    model = StorageBoxKind
    extra_context = {'menu': ['StorageBoxKind', 'storage'], 'bread': [_('StorageBoxKind'), _('Storage')]}


class StorageBoxKindCreate(StorageBoxKindUrl, GenCreate):
    model = StorageBoxKind
    form_class = StorageBoxKindForm


class StorageBoxKindCreateModal(GenCreateModal, StorageBoxKindCreate):
    pass


class StorageBoxKindUpdate(StorageBoxKindUrl, GenUpdate):
    model = StorageBoxKind
    form_class = StorageBoxKindForm


class StorageBoxKindUpdateModal(GenUpdateModal, StorageBoxKindUpdate):
    pass


class StorageBoxKindDelete(StorageBoxKindUrl, GenDelete):
    model = StorageBoxKind


class StorageBoxKindSubList(StorageBoxKindUrl, GenList):
    model = StorageBoxKind
    show_details = False
    extra_context = {'menu': ['StorageBoxKind', 'people'], 'bread': [_('StorageBoxKind'), _('People')]}

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(xxxxxxx__pk=pk)
        return limit


class StorageBoxKindDetails(StorageBoxKindUrl, GenDetail):
    model = StorageBoxKind
    groups = StorageBoxKindForm.__groups_details__()


class StorageBoxKindDetailModal(GenDetailModal, StorageBoxKindDetails):
    pass


# ###########################################
# StorageOperator
class StorageOperatorList(GenList):
    model = StorageOperator
    extra_context = {'menu': ['storage', 'storageoperator'], 'bread': [_('Storage'), _('StorageOperator')]}


class StorageOperatorCreate(GenCreate, GenCreateBridge):
    model = StorageOperator
    form_class = StorageOperatorForm

    def form_valid(self, form):
        field = 'codenerix_external_field'
        model = StorageOperator
        related_field = 'storage_operator'
        error_message = [
            _("The selected entry is already a operator, select another entry!"),
            _("The selected entry is not available anymore, please, try again!")
        ]

        external = self.request.POST.get('codenerix_external_field', None)
        password1 = self.request.POST.get('password1', None)
        password2 = self.request.POST.get('password2', None)
        if external is None:
            errors = form._errors.setdefault("codenerix_external_field", ErrorList())
            errors.append(_("Not related to a user (E1)"))
            return super(StorageOperatorCreate, self).form_invalid(form)
        else:
            model_tmp = None
            for related in self.model._meta.related_objects:
                related_model = related.related_model
                try:
                    if related_model._meta.get_field('storage_operator'):
                        model_tmp = related_model
                        break
                except FieldDoesNotExist:
                    pass
            if model_tmp:
                operator = model_tmp.objects.filter(pk=external).first()
                if operator is None or operator.user is None:
                    errors = form._errors.setdefault("codenerix_external_field", ErrorList())
                    errors.append(_("Not related to a user (E2)"))
                    return super(StorageOperatorCreate, self).form_invalid(form)
            else:
                errors = form._errors.setdefault("codenerix_external_field", ErrorList())
                errors.append(_("Not related to a user (E3)"))
                return super(StorageOperatorCreate, self).form_invalid(form)

        if password1 is None or password2 is None:
            errors = form._errors.setdefault("password1", ErrorList())
            errors.append(_("Passwords required"))
            return super(StorageOperatorCreate, self).form_invalid(form)
        if password1 != password2:
            errors = form._errors.setdefault("password1", ErrorList())
            errors.append(_("Passwords do not match"))
            return super(StorageOperatorCreate, self).form_invalid(form)

        try:
            # python 2.7
            operator.user.last_name = hashlib.sha1(password1.encode()).hexdigest()[:30]
        except TypeError:
            # python 3.x
            password1_str = bytes(password1, encoding='utf-8')
            operator.user.last_name = hashlib.sha1(password1_str.encode()).hexdigest()[:30]
        operator.user.save()
        return self.form_valid_bridge(form, field, model, related_field, error_message)


class StorageOperatorCreateModal(GenCreateModal, StorageOperatorCreate):
    pass


class StorageOperatorUpdate(GenUpdate, GenUpdateBridge):
    model = StorageOperator
    form_class = StorageOperatorForm

    def get_form(self, form_class=None):
        form = super(StorageOperatorUpdate, self).get_form(form_class)
        # initial external field
        form.fields['codenerix_external_field'].initial = form.instance.external
        return form

    def form_valid(self, form):
        field = 'codenerix_external_field'
        model = StorageOperator
        related_field = 'storage_operator'
        error_message = [
            _("The selected entry is not available anymore, please, try again!")
        ]
        external = self.request.POST.get('codenerix_external_field', None)
        password1 = self.request.POST.get('password1', None)
        password2 = self.request.POST.get('password2', None)

        if external is None:
            errors = form._errors.setdefault("codenerix_external_field", ErrorList())
            errors.append(_("Not related to a user (E1)"))
            return super(StorageOperatorUpdate, self).form_invalid(form)
        else:
            model_tmp = None
            for related in self.model._meta.related_objects:
                related_model = related.related_model
                try:
                    if related_model._meta.get_field('storage_operator'):
                        model_tmp = related_model
                        break
                except FieldDoesNotExist:
                    pass
            if model_tmp:
                operator = model_tmp.objects.filter(pk=external).first()
                if operator is None or operator.user is None:
                    errors = form._errors.setdefault("codenerix_external_field", ErrorList())
                    errors.append(_("Not related to a user (E2)"))
                    return super(StorageOperatorUpdate, self).form_invalid(form)
            else:
                errors = form._errors.setdefault("codenerix_external_field", ErrorList())
                errors.append(_("Not related to a user (E3)"))
                return super(StorageOperatorUpdate, self).form_invalid(form)

        if password1 != password2:
            errors = form._errors.setdefault("password1", ErrorList())
            errors.append(_("Passwords do not match"))
            return super(StorageOperatorUpdate, self).form_invalid(form)

        if password1:
            operator.user.last_name = hashlib.sha1(password1.encode()).hexdigest()[:30]
            operator.user.save()
        return self.form_valid_bridge(form, field, model, related_field, error_message)


class StorageOperatorUpdateModal(GenUpdateModal, StorageOperatorUpdate):
    pass


class StorageOperatorDelete(GenDelete):
    model = StorageOperator


class StorageOperatorSubList(GenList):
    model = StorageOperator
    extra_context = {'menu': ['storage', 'storageoperator'], 'bread': [_('Storage'), _('StorageOperator')]}

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(person__pk=pk)
        return limit


class StorageOperatorDetails(GenDetail):
    model = StorageOperator
    groups = StorageOperatorForm.__groups_details__()


class StorageOperatorDetailModal(GenDetailModal, StorageOperatorDetails):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-26 08:34
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0023_auto_20180414_1252'),
    ]

    operations = [
        migrations.AddField(
            model_name='inventory',
            name='storage',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='inventorys', to='codenerix_storages.Storage', verbose_name='Storage'),
        ),
        migrations.AddField(
            model_name='inventory',
            name='zone',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='inventorys', to='codenerix_storages.StorageZone', verbose_name='Zone'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-19 09:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0008_auto_20180119_1029'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='inventory',
            name='closed',
        ),
        migrations.RemoveField(
            model_name='inventory',
            name='opened',
        ),
        migrations.AddField(
            model_name='inventory',
            name='end',
            field=models.DateTimeField(blank=True, null=True, verbose_name='End'),
        ),
        migrations.AddField(
            model_name='inventory',
            name='start',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Start'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-13 14:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0019_distribution_distributionline'),
    ]

    operations = [
        migrations.AddField(
            model_name='inventoryinline',
            name='notes',
            field=models.TextField(blank=True, null=True, verbose_name='Notes'),
        ),
        migrations.AddField(
            model_name='inventoryline',
            name='notes',
            field=models.TextField(blank=True, null=True, verbose_name='Notes'),
        ),
        migrations.AddField(
            model_name='inventoryoutline',
            name='notes',
            field=models.TextField(blank=True, null=True, verbose_name='Notes'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-storages
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django import forms
from django.utils.translation import ugettext_lazy as _

from codenerix.forms import GenModelForm
from codenerix.widgets import MultiStaticSelect
from codenerix_extensions.helpers import get_external_model
from codenerix_storages.models import Storage, StorageZone
from codenerix_storages.models import StorageBox, StorageBoxStructure, StorageBoxKind, StorageOperator


class StorageForm(GenModelForm):

    class Meta:
        model = Storage
        exclude = []
        autofill = {
            'region': ['select', 3, "CDNX_ext_location_regions_foreign"],
            'country': ['select', 3, "CDNX_ext_location_country_foreign"],
            'province': ['select', 3, "CDNX_ext_location_provinces_foreign"],
            'city': ['select', 3, "CDNX_ext_location_citys_foreign"],
            }

    def __groups__(self):
        g = [
            (
                _('Details'), 12,
                ['name', 6],
            ),
            (
                _('Address'), 12,
                ['country', 3],
                ['region', 3],
                ['province', 3],
                ['city', 3],
                ['town', 9],
                ['zipcode', 3],
                ['address', 9],
                ['phone', 3],
            )
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (
                _('Details'), 12,
                ['name', 6],
            ),
            (
                _('Address'), 12,
                ['country', 3],
                ['region', 3],
                ['province', 3],
                ['city', 3],
                ['town', 9],
                ['zipcode', 3],
                ['address', 9],
                ['phone', 3],
            )
        ]
        return g


class StorageZoneForm(GenModelForm):
    class Meta:
        model = StorageZone
        exclude = []

    def __groups__(self):
        g = [
            (
                _('Details'), 12,
                ['storage', 6],
                ['name', 6],
                ['salable', 6],
            )
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (
                _('Details'), 12,
                ['storage', 6],
                ['name', 6],
                ['salable', 6],
            )
        ]
        return g


class StorageZoneOwnForm(GenModelForm):
    class Meta:
        model = StorageZone
        exclude = ['storage', ]

    def __groups__(self):
        g = [
            (
                _('Details'), 12,
                ['name', 6],
                ['salable', 6],
            )
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (
                _('Details'), 12,
                ['storage', 6],
                ['name', 6],
                ['salable', 6],
            )
        ]
        return g


class StorageBoxForm(GenModelForm):
    class Meta:
        model = StorageBox
        exclude = ['weight', ]

    def __groups__(self):
        g = [
            (
                _('Details'), 12,
                ['box_structure', 6],
                ['box_kind', 6],
                ['name', 6],
            )
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
            (
                _('Details'), 12,
                ['box_structure', 6],
                ['box_kind', 6],
                ['name', 6],
                ['weight', 6],
            )
        ]
        return g


class StorageBoxFormUpdate(GenModelForm):
    class Meta:
        model = StorageBox
        exclude = []

    def __groups__(self):
        g = [
            (
                _('Details'), 12,
                ['box_structure', 6],
                ['box_kind', 6],
                ['name', 6],
                ['weight', 6, {'extra': ['ng-disabled=true']}],
            )
        ]
        return g


class StorageBoxStructureForm(GenModelForm):
    class Meta:
        model = StorageBoxStructure
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['zone', 6],
                ['box_structure', 6],
                ['length', 6],
                ['width', 6],
                ['heigth', 6],
                ['weight', 6],
                ['max_weight', 6],
                ['name', 6],
            )
        ]

    @staticmethod
    def __groups_details__():
        g = [
            (
                _('Details'), 12,
                ['zone', 6],
                ['box_structure', 6],
                ['length', 6],
                ['width', 6],
                ['heigth', 6],
                ['weight', 6],
                ['max_weight', 6],
                ['name', 6],
            )
        ]
        return g


class StorageBoxKindForm(GenModelForm):
    class Meta:
        model = StorageBoxKind
        exclude = []

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['length', 6],
                ['width', 6],
                ['heigth', 6],
                ['weight', 6],
                ['max_weight', 6],
                ['name', 6],
            )
        ]

    @staticmethod
    def __groups_details__():
        return [
            (
                _('Details'), 12,
                ['length', 6],
                ['width', 6],
                ['heigth', 6],
                ['weight', 6],
                ['max_weight', 6],
                ['name', 6],
            )
        ]


class StorageOperatorForm(GenModelForm):
    codenerix_external_field = forms.ModelChoiceField(
        label=StorageOperator.foreignkey_external()['label'],
        queryset=get_external_model(StorageOperator).objects.all()
    )
    zone = forms.ModelMultipleChoiceField(
        queryset=StorageZone.objects.all().order_by('name'),
        label=_('Zones'),
        required=False,
        widget=MultiStaticSelect(
            attrs={'manytomany': True, }
        )
    )

    password1 = forms.CharField(label=_("Pin for storage software"), min_length=4, widget=forms.PasswordInput, required=False)
    password2 = forms.CharField(label=_("Confirm pin"), min_length=4, widget=forms.PasswordInput, required=False)

    class Meta:
        model = StorageOperator
        exclude = []
        autofill = {
            'codenerix_external_field': ['select', 3, StorageOperator.foreignkey_external()['related']],
        }

    def __groups__(self):
        return [
            (
                _('Details'), 12,
                ['codenerix_external_field', 6],
                ['zone', 4],
                ['enable', 2],
                ['password1', 6],
                ['password2', 6],
            )
        ]

    @staticmethod
    def __groups_details__():
        return [
            (
                _('Details'), 12,
                ['codenerix_external_field', 6],
                ['zone', 4],
                ['enable', 2],
            )
        ]

    def clean(self):
        cleaned_data = super(StorageOperatorForm, self).clean()

        if cleaned_data.get('password1') != cleaned_data.get('password2'):
            del cleaned_data['password1']
            del cleaned_data['password2']
            raise forms.ValidationError(_("Passwords do not match"))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-30 17:09
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0025_auto_20180426_1035'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='lineoutgoingalbaran',
            name='prepare_user',
        ),
        migrations.AddField(
            model_name='outgoingalbaran',
            name='inventory',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='outgoing_albarans', to='codenerix_storages.InventoryOut', verbose_name='Inventory'),
        ),
        migrations.AlterField(
            model_name='outgoingalbaran',
            name='request_stock',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='outgoing_albarans', to='codenerix_storages.RequestStock', verbose_name='Request stock'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-19 11:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0014_inventoryinline_purchasesorder'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inventoryinline',
            name='purchasesorder',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='inventory_lines', to='codenerix_invoicing.PurchasesOrder', verbose_name='Inventory line'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-25 07:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0009_auto_20180119_1034'),
    ]

    operations = [
        migrations.AddField(
            model_name='inventoryline',
            name='caducity',
            field=models.DateField(blank=True, default=None, null=True, verbose_name='Caducity'),
        ),
        migrations.AddField(
            model_name='inventoryline',
            name='product_unique_value',
            field=models.CharField(blank=True, default=None, editable=False, max_length=80, null=True, verbose_name='Product Unique Value'),
        ),
        migrations.AlterField(
            model_name='inventory',
            name='end',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Ends'),
        ),
        migrations.AlterField(
            model_name='inventory',
            name='start',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Starts'),
        ),
        migrations.AlterField(
            model_name='inventoryline',
            name='product_unique',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventorylines', to='codenerix_products.ProductUnique', verbose_name='Product Unique'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-storages
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import url
from codenerix_storages.views import StorageList, StorageCreate, StorageCreateModal, StorageUpdate, StorageUpdateModal, StorageDelete, StorageDetails
from codenerix_storages.views import StorageZoneList, StorageZoneCreate, StorageZoneCreateModal, StorageZoneUpdate, StorageZoneUpdateModal, StorageZoneDelete, StorageZoneSubList, StorageZoneDetail, StorageZoneDetailModal, OwnStorageZoneCreateModal, OwnStorageZoneUpdateModal
from codenerix_storages.views import StorageBoxList, StorageBoxCreate, StorageBoxCreateModal, StorageBoxDetail, StorageBoxUpdate, StorageBoxUpdateModal, StorageBoxDelete, StorageBoxSubList, StorageBoxDetailModal, StorageBoxForeign
from codenerix_storages.views import StorageBoxStructureList, StorageBoxStructureCreate, StorageBoxStructureCreateModal, StorageBoxStructureUpdate, StorageBoxStructureUpdateModal, StorageBoxStructureDelete, StorageBoxStructureSubList, StorageBoxStructureDetails, StorageBoxStructureDetailModal
from codenerix_storages.views import StorageBoxKindList, StorageBoxKindCreate, StorageBoxKindCreateModal, StorageBoxKindUpdate, StorageBoxKindUpdateModal, StorageBoxKindDelete, StorageBoxKindSubList, StorageBoxKindDetails, StorageBoxKindDetailModal
from codenerix_storages.views import StorageOperatorList, StorageOperatorCreate, StorageOperatorCreateModal, StorageOperatorUpdate, StorageOperatorUpdateModal, StorageOperatorDelete, StorageOperatorSubList, StorageOperatorDetails, StorageOperatorDetailModal

urlpatterns = [
    url(r'^storages$', StorageList.as_view(), name='CDNX_storages_storages_list'),
    url(r'^storages/add$', StorageCreate.as_view(), name='CDNX_storages_storages_add'),
    url(r'^storages/addmodal$', StorageCreateModal.as_view(), name='CDNX_storages_storages_addmodal'),
    url(r'^storages/(?P<pk>\w+)$', StorageDetails.as_view(), name='CDNX_storages_storages_details'),
    url(r'^storages/(?P<pk>\w+)/edit$', StorageUpdate.as_view(), name='CDNX_storages_storages_edit'),
    url(r'^storages/(?P<pk>\w+)/editmodal$', StorageUpdateModal.as_view(), name='CDNX_storages_storages_editmodal'),
    url(r'^storages/(?P<pk>\w+)/delete$', StorageDelete.as_view(), name='CDNX_storages_storages_delete'),

    # StorageZone
    url(r'^storagezones$', StorageZoneList.as_view(), name='CDNX_storages_storagezones_list'),
    url(r'^storagezones/add$', StorageZoneCreate.as_view(), name='CDNX_storages_storagezones_add'),
    url(r'^storagezones/addmodal$', StorageZoneCreateModal.as_view(), name='CDNX_storages_storagezones_addmodal'),
    url(r'^storagezones/(?P<pk>\w+)$', StorageZoneDetail.as_view(), name='CDNX_storages_storagezones_detail'),
    url(r'^storagezones/(?P<pk>\w+)/edit$', StorageZoneUpdate.as_view(), name='CDNX_storages_storagezones_edit'),
    url(r'^storagezones/(?P<pk>\w+)/editmodal$', StorageZoneUpdateModal.as_view(), name='CDNX_storages_storagezones_editmodal'),
    url(r'^storagezones/(?P<pk>\w+)/delete$', StorageZoneDelete.as_view(), name='CDNX_storages_storagezones_delete'),
    url(r'^storagezones/(?P<pk>\w+)/sublist$', StorageZoneSubList.as_view(), name='CDNX_storages_storagezones_sublist'),
    url(r'^storagezones/(?P<pk>\w+)/sublist/add$', StorageZoneCreateModal.as_view(), name='CDNX_storages_storagezones_sublist_add'),
    url(r'^storagezones/(?P<pk>\w+)/sublist/addmodal$', OwnStorageZoneCreateModal.as_view(), name='CDNX_storages_storagezones_sublist_addmodal'),
    url(r'^storagezones/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', StorageZoneDetailModal.as_view(), name='CDNX_storages_storagezones_sublist_details'),
    url(r'^storagezones/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', StorageZoneUpdateModal.as_view(), name='CDNX_storages_storagezones_sublist_edit'),
    url(r'^storagezones/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', OwnStorageZoneUpdateModal.as_view(), name='CDNX_storages_storagezones_sublist_editmodal'),
    url(r'^storagezones/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', StorageZoneDelete.as_view(), name='CDNX_storages_storagezones_sublist_delete'),

    # StorageBox
    url(r'^storageboxs$', StorageBoxList.as_view(), name='CDNX_storages_storageboxs_list'),
    url(r'^storageboxs/add$', StorageBoxCreate.as_view(), name='CDNX_storages_storageboxs_add'),
    url(r'^storageboxs/addmodal$', StorageBoxCreateModal.as_view(), name='CDNX_storages_storageboxs_addmodal'),
    url(r'^storageboxs/foreign/(?P<search>[\w\W]+|\*)$', StorageBoxForeign.as_view(), name='CDNX_storages_storageboxs_foreign'),
    url(r'^storageboxs/(?P<pk>\w+)$', StorageBoxDetail.as_view(), name='CDNX_storages_storagezones_detail'),
    url(r'^storageboxs/(?P<pk>\w+)/edit$', StorageBoxUpdate.as_view(), name='CDNX_storages_storageboxs_edit'),
    url(r'^storageboxs/(?P<pk>\w+)/editmodal$', StorageBoxUpdateModal.as_view(), name='CDNX_storages_storageboxs_editmodal'),
    url(r'^storageboxs/(?P<pk>\w+)/delete$', StorageBoxDelete.as_view(), name='CDNX_storages_storageboxs_delete'),

    url(r'^storageboxs/(?P<pk>\w+)/sublist$', StorageBoxSubList.as_view(), name='CDNX_storages_storageboxs_sublist'),
    url(r'^storageboxs/(?P<pk>\w+)/sublist/add$', StorageBoxCreateModal.as_view(), name='CDNX_storages_storageboxs_sublist_add'),
    url(r'^storageboxs/(?P<pk>\w+)/sublist/addmodal$', StorageBoxCreateModal.as_view(), name='CDNX_storages_storageboxs_sublist_addmodal'),
    url(r'^storageboxs/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', StorageBoxDetailModal.as_view(), name='CDNX_storages_storageboxs_sublist_details'),
    url(r'^storageboxs/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', StorageBoxUpdateModal.as_view(), name='CDNX_storages_storageboxs_sublist_edit'),
    url(r'^storageboxs/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', StorageBoxUpdateModal.as_view(), name='CDNX_storages_storageboxs_sublist_editmodal'),
    url(r'^storageboxs/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', StorageBoxDelete.as_view(), name='CDNX_storages_storageboxs_sublist_delete'),

    # StorageBoxStructure
    url(r'^storageboxstructures$', StorageBoxStructureList.as_view(), name='CDNX_storages_storageboxstructures_list'),
    url(r'^storageboxstructures/add$', StorageBoxStructureCreate.as_view(), name='CDNX_storages_storageboxstructures_add'),
    url(r'^storageboxstructures/addmodal$', StorageBoxStructureCreateModal.as_view(), name='CDNX_storages_storageboxstructures_addmodal'),
    url(r'^storageboxstructures/(?P<pk>\w+)$', StorageBoxStructureDetails.as_view(), name='CDNX_storages_storageboxstructures_details'),
    url(r'^storageboxstructures/(?P<pk>\w+)/edit$', StorageBoxStructureUpdate.as_view(), name='CDNX_storages_storageboxstructures_edit'),
    url(r'^storageboxstructures/(?P<pk>\w+)/editmodal$', StorageBoxStructureUpdateModal.as_view(), name='CDNX_storages_storageboxstructures_editmodal'),
    url(r'^storageboxstructures/(?P<pk>\w+)/delete$', StorageBoxStructureDelete.as_view(), name='CDNX_storages_storageboxstructures_delete'),
    url(r'^storageboxstructures/(?P<pk>\w+)/sublist$', StorageBoxStructureSubList.as_view(), name='CDNX_storages_storageboxstructures_sublist'),
    url(r'^storageboxstructures/(?P<pk>\w+)/sublist/add$', StorageBoxStructureCreateModal.as_view(), name='CDNX_storages_storageboxstructures_sublist_add'),
    url(r'^storageboxstructures/(?P<pk>\w+)/sublist/addmodal$', StorageBoxStructureCreateModal.as_view(), name='CDNX_storages_storageboxstructures_sublist_addmodal'),
    url(r'^storageboxstructures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', StorageBoxStructureDetailModal.as_view(), name='CDNX_storages_storageboxstructures_sublist_details'),
    url(r'^storageboxstructures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', StorageBoxStructureUpdateModal.as_view(), name='CDNX_storages_storageboxstructures_sublist_edit'),
    url(r'^storageboxstructures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', StorageBoxStructureUpdateModal.as_view(), name='CDNX_storages_storageboxstructures_sublist_editmodal'),
    url(r'^storageboxstructures/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', StorageBoxStructureDelete.as_view(), name='CDNX_storages_storageboxstructures_sublist_delete'),

    # StorageBoxKind
    url(r'^storageboxkinds$', StorageBoxKindList.as_view(), name='CDNX_storages_storageboxkinds_list'),
    url(r'^storageboxkinds/add$', StorageBoxKindCreate.as_view(), name='CDNX_storages_storageboxkinds_add'),
    url(r'^storageboxkinds/addmodal$', StorageBoxKindCreateModal.as_view(), name='CDNX_storages_storageboxkinds_addmodal'),
    url(r'^storageboxkinds/(?P<pk>\w+)$', StorageBoxKindDetails.as_view(), name='CDNX_storages_storageboxkinds_details'),
    url(r'^storageboxkinds/(?P<pk>\w+)/edit$', StorageBoxKindUpdate.as_view(), name='CDNX_storages_storageboxkinds_edit'),
    url(r'^storageboxkinds/(?P<pk>\w+)/editmodal$', StorageBoxKindUpdateModal.as_view(), name='CDNX_storages_storageboxkinds_editmodal'),
    url(r'^storageboxkinds/(?P<pk>\w+)/delete$', StorageBoxKindDelete.as_view(), name='CDNX_storages_storageboxkinds_delete'),
    url(r'^storageboxkinds/(?P<pk>\w+)/sublist$', StorageBoxKindSubList.as_view(), name='CDNX_storages_storageboxkinds_sublist'),
    url(r'^storageboxkinds/(?P<pk>\w+)/sublist/add$', StorageBoxKindCreateModal.as_view(), name='CDNX_storages_storageboxkinds_sublist_add'),
    url(r'^storageboxkinds/(?P<pk>\w+)/sublist/addmodal$', StorageBoxKindCreateModal.as_view(), name='CDNX_storages_storageboxkinds_sublist_addmodal'),
    url(r'^storageboxkinds/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', StorageBoxKindDetailModal.as_view(), name='CDNX_storages_storageboxkinds_sublist_details'),
    url(r'^storageboxkinds/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', StorageBoxKindUpdateModal.as_view(), name='CDNX_storages_storageboxkinds_sublist_edit'),
    url(r'^storageboxkinds/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', StorageBoxKindUpdateModal.as_view(), name='CDNX_storages_storageboxkinds_sublist_editmodal'),
    url(r'^storageboxkinds/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', StorageBoxKindDelete.as_view(), name='CDNX_storages_storageboxkinds_sublist_delete'),

    url(r'^storageoperators$', StorageOperatorList.as_view(), name='CDNX_storages_storageoperators_list'),
    url(r'^storageoperators/add$', StorageOperatorCreate.as_view(), name='CDNX_storages_storageoperators_add'),
    url(r'^storageoperators/addmodal$', StorageOperatorCreateModal.as_view(), name='CDNX_storages_storageoperators_addmodal'),
    url(r'^storageoperators/(?P<pk>\w+)$', StorageOperatorDetails.as_view(), name='CDNX_storages_storageoperators_details'),
    url(r'^storageoperators/(?P<pk>\w+)/edit$', StorageOperatorUpdate.as_view(), name='CDNX_storages_storageoperators_edit'),
    url(r'^storageoperators/(?P<pk>\w+)/editmodal$', StorageOperatorUpdateModal.as_view(), name='CDNX_storages_storageoperators_editmodal'),
    url(r'^storageoperators/(?P<pk>\w+)/delete$', StorageOperatorDelete.as_view(), name='CDNX_storages_storageoperators_delete'),
    url(r'^storageoperators/(?P<pk>\w+)/sublist$', StorageOperatorSubList.as_view(), name='CDNX_storages_storageoperators_sublist'),
    url(r'^storageoperators/(?P<pk>\w+)/sublist/add$', StorageOperatorCreateModal.as_view(), name='CDNX_storages_storageoperators_sublist_add'),
    url(r'^storageoperators/(?P<pk>\w+)/sublist/addmodal$', StorageOperatorCreateModal.as_view(), name='CDNX_storages_storageoperators_sublist_addmodal'),
    url(r'^storageoperators/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', StorageOperatorDetailModal.as_view(), name='CDNX_storages_storageoperators_sublist_details'),
    url(r'^storageoperators/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', StorageOperatorUpdateModal.as_view(), name='CDNX_storages_storageoperators_sublist_edit'),
    url(r'^storageoperators/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/editmodal$', StorageOperatorUpdateModal.as_view(), name='CDNX_storages_storageoperators_sublist_editmodal'),
    url(r'^storageoperators/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', StorageOperatorDelete.as_view(), name='CDNX_storages_storageoperators_sublist_delete'),


]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-18 10:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0002_auto_20180118_1040'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='storage',
            name='alias',
        ),
        migrations.AlterField(
            model_name='storagebox',
            name='box_kind',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_boxes', to='codenerix_storages.StorageBoxKind', verbose_name='Box Kind'),
        ),
        migrations.AlterField(
            model_name='storagebox',
            name='box_structure',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_boxes', to='codenerix_storages.StorageBoxStructure', verbose_name='Box Structure'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-18 17:03
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0006_auto_20180118_1724'),
    ]

    operations = [
        migrations.RenameField(
            model_name='inventoryalbaran',
            old_name='storage_operator',
            new_name='operator',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-18 16:24
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0006_auto_20180118_1126'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('codenerix_storages', '0005_auto_20180118_1417'),
    ]

    operations = [
        migrations.CreateModel(
            name='IncomingAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('reception_date', models.DateTimeField(blank=True, null=True, verbose_name='Reception Date')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Inventory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=80, verbose_name='Name')),
                ('opened', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Date')),
                ('closed', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Date')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=80, verbose_name='Name')),
                ('opened', models.DateTimeField(default=django.utils.timezone.now, editable=False, verbose_name='Date')),
                ('closed', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='Date')),
                ('inventory', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventoryalbarans', to='codenerix_storages.Inventory', verbose_name='Inventory')),
                ('storage_operator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventoryalbarans', to='codenerix_storages.StorageOperator', verbose_name='Storage Operator')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryAlbaranLine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('inventory_albaran', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventoryalbaranlines', to='codenerix_storages.InventoryAlbaran', verbose_name='Inventory Albaran')),
                ('product_unique', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventoryalbaranlines', to='codenerix_products.ProductUnique', verbose_name='Product')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LineIncomingAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('quantity', models.FloatField(verbose_name='Quantity')),
                ('box', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_ingoming_albarans', to='codenerix_storages.StorageBox', verbose_name='Box')),
                ('incoming_albaran', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_incoming_albarans', to='codenerix_storages.IncomingAlbaran', verbose_name='Incoming Albaran')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LineOutgoingAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LineRequestStock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('quantity', models.FloatField(verbose_name='Quantity')),
                ('product_final', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_request_stock', to='codenerix_products.ProductFinal', verbose_name='Product')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OutgoingAlbaran',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('prepare_date', models.DateTimeField(verbose_name='Prepare Date')),
                ('outgoing_date', models.DateTimeField(blank=True, null=True, verbose_name='Outgoing Date')),
                ('estimated_date', models.DateTimeField(blank=True, null=True, verbose_name='Estimated Date')),
                ('prepare_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='outgoing_albarans', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RequestStock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('code', models.CharField(max_length=64, verbose_name='Code')),
                ('code_counter', models.IntegerField(editable=False, verbose_name='Code counter')),
                ('request_date', models.DateTimeField(verbose_name='Request Date')),
                ('desired_date', models.DateTimeField(verbose_name='Desired Date')),
                ('request_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='request_stocks', to=settings.AUTH_USER_MODEL)),
                ('storage_destination', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='request_stocks_dst', to='codenerix_storages.Storage', verbose_name='Storage destionation')),
                ('storage_source', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='request_stocks_src', to='codenerix_storages.Storage', verbose_name='Storage source')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='outgoingalbaran',
            name='request_stock',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='outgoing_albarans', to='codenerix_storages.RequestStock', verbose_name='Request stock'),
        ),
        migrations.AddField(
            model_name='linerequeststock',
            name='request_stock',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_request_stock', to='codenerix_storages.RequestStock', verbose_name='Request stock'),
        ),
        migrations.AddField(
            model_name='lineoutgoingalbaran',
            name='outgoing_albaran',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_outgoing_albarans', to='codenerix_storages.OutgoingAlbaran', verbose_name='Outgoing Albaran'),
        ),
        migrations.AddField(
            model_name='lineoutgoingalbaran',
            name='prepare_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_outgoing_albarans_prepare', to=settings.AUTH_USER_MODEL, verbose_name='Prepare user'),
        ),
        migrations.AddField(
            model_name='lineoutgoingalbaran',
            name='product_unique',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_outgoing_albarans', to='codenerix_products.ProductUnique', verbose_name='Outgoing Albaran'),
        ),
        migrations.AddField(
            model_name='lineoutgoingalbaran',
            name='validator_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_outgoing_albarans_validator', to=settings.AUTH_USER_MODEL, verbose_name='Validator user'),
        ),
        migrations.AddField(
            model_name='lineincomingalbaran',
            name='product_unique',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_incoming_albarans', to='codenerix_storages.OutgoingAlbaran', verbose_name='Product'),
        ),
        migrations.AddField(
            model_name='lineincomingalbaran',
            name='validator_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_incoming_albarans', to=settings.AUTH_USER_MODEL, verbose_name='Validator user'),
        ),
        migrations.AddField(
            model_name='incomingalbaran',
            name='outgoing_albaran',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='incoming_albarans', to='codenerix_storages.OutgoingAlbaran', verbose_name='Outgoing Albaran'),
        ),
        migrations.AddField(
            model_name='incomingalbaran',
            name='reception_user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='incoming_albarans', to=settings.AUTH_USER_MODEL, verbose_name='Reception user'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-14 10:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0017_cashdiary_cashmovement'),
        ('codenerix_storages', '0022_auto_20180414_0607'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='inventoryinline',
            name='purchaseslinealbaran',
        ),
        migrations.AddField(
            model_name='inventoryinline',
            name='purchaseslinealbaran',
            field=models.ManyToManyField(related_name='inventory_lines', to='codenerix_invoicing.PurchasesLineAlbaran', verbose_name='Line Albaran'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-17 15:28
from __future__ import unicode_literals

import codenerix.models_people
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('codenerix_geodata', '0002_auto_20170921_1206'),
    ]

    operations = [
        migrations.CreateModel(
            name='Storage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('alias', models.CharField(blank=True, max_length=100, null=True, verbose_name='Alias')),
                ('town', models.CharField(blank=True, max_length=250, null=True, verbose_name='Town')),
                ('zipcode', models.CharField(blank=True, max_length=6, null=True, verbose_name='Zip code')),
                ('address', models.CharField(blank=True, max_length=250, null=True, verbose_name='Address')),
                ('phone', models.CharField(blank=True, max_length=16, null=True, verbose_name='Phone')),
                ('name', models.CharField(max_length=80, verbose_name='Storage')),
                ('city', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='codenerix_storages_storage_geo_addresses', to='codenerix_geodata.City', verbose_name='City')),
                ('country', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='codenerix_storages_storage_geo_addresses', to='codenerix_geodata.Country', verbose_name='Country')),
                ('province', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='codenerix_storages_storage_geo_addresses', to='codenerix_geodata.Province', verbose_name='Province')),
                ('region', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='codenerix_storages_storage_geo_addresses', to='codenerix_geodata.Region', verbose_name='Region')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='StorageBox',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=80, verbose_name='Name')),
                ('weight', models.FloatField(blank=True, null=True, verbose_name='Weight')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='StorageBoxKind',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('length', models.FloatField(blank=True, null=True, verbose_name='Length')),
                ('width', models.FloatField(blank=True, null=True, verbose_name='Width')),
                ('heigth', models.FloatField(blank=True, null=True, verbose_name='Heigth')),
                ('weight', models.FloatField(blank=True, null=True, verbose_name='Weight')),
                ('max_weight', models.FloatField(blank=True, null=True, verbose_name='Weight')),
                ('name', models.CharField(max_length=80, verbose_name='Name')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='StorageBoxStructure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('length', models.FloatField(blank=True, null=True, verbose_name='Length')),
                ('width', models.FloatField(blank=True, null=True, verbose_name='Width')),
                ('heigth', models.FloatField(blank=True, null=True, verbose_name='Heigth')),
                ('weight', models.FloatField(blank=True, null=True, verbose_name='Weight')),
                ('max_weight', models.FloatField(blank=True, null=True, verbose_name='Weight')),
                ('name', models.CharField(max_length=80, verbose_name='Name')),
                ('box_structure', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='storage_boxes_structure', to='codenerix_storages.StorageBoxStructure', verbose_name='Structure box')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='StorageOperator',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('enable', models.BooleanField(default=True, verbose_name='Enable')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
            bases=(codenerix.models_people.GenRole, models.Model),
        ),
        migrations.CreateModel(
            name='StorageZone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('name', models.CharField(max_length=80, verbose_name='Zone')),
                ('salable', models.BooleanField(default=True, verbose_name='Salable')),
                ('storage', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_zones', to='codenerix_storages.Storage', verbose_name='Storage')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='storageoperator',
            name='zone',
            field=models.ManyToManyField(related_name='storage_operators', to='codenerix_storages.StorageZone', verbose_name='Zones'),
        ),
        migrations.AddField(
            model_name='storageboxstructure',
            name='zone',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='storage_boxes_structure', to='codenerix_storages.StorageZone', verbose_name='Zone'),
        ),
        migrations.AddField(
            model_name='storagebox',
            name='box_kind',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='storage_boxes', to='codenerix_storages.StorageBoxKind', verbose_name='Box Kind'),
        ),
        migrations.AddField(
            model_name='storagebox',
            name='box_structure',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='storage_boxes', to='codenerix_storages.StorageBoxStructure', verbose_name='Box Structure'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-07-08 05:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0029_auto_20180430_1924'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lineoutgoingalbaran',
            name='product_unique',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='line_outgoing_albarans', to='codenerix_products.ProductUnique', verbose_name='Product Unique'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-18 09:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_storages', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='storageboxkind',
            name='max_weight',
            field=models.FloatField(blank=True, null=True, verbose_name='Max weight'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-01-19 09:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_products', '0006_auto_20180118_1126'),
        ('codenerix_storages', '0007_auto_20180118_1803'),
    ]

    operations = [
        migrations.CreateModel(
            name='InventoryLine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('quantity', models.FloatField(default=1.0, verbose_name='Quantity')),
                ('box', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventorys', to='codenerix_storages.StorageBox', verbose_name='Box')),
            ],
            options={
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
                'abstract': False,
            },
        ),
        migrations.RemoveField(
            model_name='inventoryalbaran',
            name='inventory',
        ),
        migrations.RemoveField(
            model_name='inventoryalbaran',
            name='operator',
        ),
        migrations.RemoveField(
            model_name='inventoryalbaranline',
            name='inventory_albaran',
        ),
        migrations.RemoveField(
            model_name='inventoryalbaranline',
            name='product_unique',
        ),
        migrations.AlterField(
            model_name='inventory',
            name='closed',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Date'),
        ),
        migrations.AlterField(
            model_name='inventory',
            name='opened',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Date'),
        ),
        migrations.DeleteModel(
            name='InventoryAlbaran',
        ),
        migrations.DeleteModel(
            name='InventoryAlbaranLine',
        ),
        migrations.AddField(
            model_name='inventoryline',
            name='inventory',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventorys', to='codenerix_storages.Inventory', verbose_name='Inventory'),
        ),
        migrations.AddField(
            model_name='inventoryline',
            name='operator',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventorys', to='codenerix_storages.StorageOperator', verbose_name='Storage Operator'),
        ),
        migrations.AddField(
            model_name='inventoryline',
            name='product_final',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventorylines', to='codenerix_products.ProductFinal', verbose_name='Product Final'),
        ),
        migrations.AddField(
            model_name='inventoryline',
            name='product_unique',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='storage_inventorylines', to='codenerix_products.ProductUnique', verbose_name='Product Unique'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-27 07:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_invoicing', '0014_auto_20180227_0858'),
        ('codenerix_storages', '0017_inventoryoutline_caducity'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='inventoryoutline',
            name='caducity',
        ),
        migrations.AddField(
            model_name='inventoryinline',
            name='purchaseslinealbaran',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='inventory_lines', to='codenerix_invoicing.PurchasesLineAlbaran', verbose_name='Line Albaran'),
        ),
        migrations.AlterField(
            model_name='inventoryinline',
            name='inventory',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='inventory_lines', to='codenerix_storages.InventoryIn', verbose_name='Inventory'),
        ),
        migrations.AlterField(
            model_name='inventoryinline',
            name='purchasesorder',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='inventory_lines', to='codenerix_invoicing.PurchasesOrder', verbose_name='Order'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-storages
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.db import models
from django.utils.translation import ugettext_lazy as _

from codenerix.models import GenInterface, CodenerixModel
from codenerix.models_people import GenRole

from codenerix_extensions.helpers import get_external_method
from codenerix_geodata.models import GeoAddress
from codenerix_storages.settings import CDNX_STORAGE_PERMISSIONS


class Storage(GeoAddress, CodenerixModel):
    name = models.CharField(_("Storage"), max_length=80, null=False, blank=False)

    def __fields__(self, info):
        fields = super(Storage, self).__fields__(info)
        fields.insert(0, ('name', _('Storage')))
        return fields

    def __str__(self):
        return u"{}".format(self.name)

    def __unicode__(self):
        return self.__str__()


# zonas del almacen
class StorageZone(CodenerixModel):
    storage = models.ForeignKey(Storage, on_delete=models.CASCADE, related_name='storage_zones', verbose_name=_("Storage"), null=False, blank=False)
    name = models.CharField(_("Zone"), max_length=80, null=False, blank=False)
    salable = models.BooleanField(_('Salable'), default=True)

    def __fields__(self, info):
        fields = []
        fields.append(('name', _('Zone'), 100))
        fields.append(('storage', _('Storage'), 100))
        fields.append(('salable', _('Salable'), 100))
        return fields

    def __str__(self):
        return u"{}".format(self.name)

    def __unicode__(self):
        return self.__str__()

    def lock_delete(self):
        if self.storage_operators.exists():
            return _("Cannot delete storage zone model, relationship between storage zone model and storage operator")
        elif self.storage_boxes_structure.exists():
            return _("Cannot delete storage zone model, relationship between storage zone model and storage box structure")
        else:
            return super(StorageZone, self).lock_delete()


class StorageBoxStructure(CodenerixModel):
    zone = models.ForeignKey(StorageZone, on_delete=models.CASCADE, related_name='storage_boxes_structure', verbose_name=_("Zone"), null=True, blank=True)
    box_structure = models.ForeignKey('StorageBoxStructure', on_delete=models.CASCADE, related_name='storage_boxes_structure', verbose_name=_("Structure box"), null=True, blank=True)
    length = models.FloatField(_('Length'), blank=True, null=True)
    width = models.FloatField(_('Width'), blank=True, null=True)
    heigth = models.FloatField(_('Heigth'), blank=True, null=True)
    # peso de las cajas y estructuras de cajas contenidas
    weight = models.FloatField(_('Weight'), blank=True, null=True)
    max_weight = models.FloatField(_('Max weight'), blank=True, null=True)
    name = models.CharField(_("Name"), max_length=80, null=False, blank=False)

    def __str__(self):
        return u"{}".format(self.name)

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('zone', _('Zone'), 100))
        fields.append(('box_structure', _('Box structure'), 100))
        fields.append(('name', _('Name'), 100))
        fields.append(('width', _('Width')))
        fields.append(('heigth', _('Heigth')))
        fields.append(('weight', _('Weight')))
        fields.append(('max_weight', _('Max weight')))
        return fields

    def lock_delete(self):
        if self.storage_boxes_structure.exists():
            return _("Cannot delete storage zone model, related to himself")
        elif self.storage_boxes.exists():
            return _("Cannot delete storage zone model, relationship between storage structure model and storage box")
        else:
            return super(StorageBoxStructure, self).lock_delete()


class StorageBoxKind(CodenerixModel):
    length = models.FloatField(_('Length'), blank=True, null=True)
    width = models.FloatField(_('Width'), blank=True, null=True)
    heigth = models.FloatField(_('Heigth'), blank=True, null=True)
    # caja vacía
    weight = models.FloatField(_('Weight'), blank=True, null=True)
    max_weight = models.FloatField(_('Max weight'), blank=True, null=True)
    name = models.CharField(_("Name"), max_length=80, null=False, blank=False)

    def __str__(self):
        return u"{}".format(self.name)

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('name', _("Name")))
        fields.append(('length', _('Length')))
        fields.append(('width', _('Width')))
        fields.append(('heigth', _('Heigth')))
        fields.append(('weight', _('Weight')))
        return fields

    def lock_delete(self):
        if self.storage_boxes.exists():
            return _("Cannot delete storage zone model, relationship between storage box kind model and storage box")
        else:
            return super(StorageBoxKind, self).lock_delete()


class StorageBox(CodenerixModel):
    box_structure = models.ForeignKey(StorageBoxStructure, on_delete=models.CASCADE, related_name='storage_boxes', verbose_name=_("Box Structure"), null=False, blank=False)
    box_kind = models.ForeignKey(StorageBoxKind, on_delete=models.CASCADE, related_name='storage_boxes', verbose_name=_("Box Kind"), null=False, blank=False)
    name = models.CharField(_("Name"), max_length=80, null=False, blank=False)
    # peso de los productos contenidos
    weight = models.FloatField(_('Weight'), blank=False, null=False, default=0)

    def __str__(self):
        return u"{}".format(self.name)

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('name', _('Name'), 100))
        fields.append(('box_structure', _('Box Structure'), 100))
        fields.append(('box_kind', _('Box Kind'), 100))
        fields.append(('weight', _('Weight'), 100))
        return fields


# ############################
class ABSTRACT_GenStorageOperator(models.Model):  # META: Abstract class

    class Meta(object):
        abstract = True


class StorageOperator(GenRole, CodenerixModel):
    class CodenerixMeta:
        abstract = ABSTRACT_GenStorageOperator
        rol_groups = {
            'StorageOperator': CDNX_STORAGE_PERMISSIONS['operator'],
        }
        rol_permissions = []
        force_methods = {
            'foreignkey_storage_operator': ('CDNX_get_fk_info_storage_operator', _('---')),
        }

    zone = models.ManyToManyField(StorageZone, related_name='storage_operators', verbose_name=_('Zones'))
    enable = models.BooleanField(_("Enable"), blank=False, null=False, default=True)

    @staticmethod
    def foreignkey_external():
        return get_external_method(StorageOperator, StorageOperator.CodenerixMeta.force_methods['foreignkey_storage_operator'][0])

    def __str__(self):
        return u"{}".format(self.pk)

    def __unicode__(self):
        return self.__str__()

    def __fields__(self, info):
        fields = []
        fields.append(('zone', _("Zones")))
        fields.append(('enable', _("Enable")))
        fields = get_external_method(StorageOperator, '__fields_storage__', info, fields)
        return fields


# puente entre los datos del almacen y 'person'
class GenStorageOperator(GenInterface, ABSTRACT_GenStorageOperator):  # META: Abstract class
    storage_operator = models.OneToOneField(StorageOperator, related_name='external', verbose_name=_("Storage operator"), null=True, on_delete=models.SET_NULL, blank=True)

    class Meta(GenInterface.Meta, ABSTRACT_GenStorageOperator.Meta):
        abstract = True

    class CodenerixMeta:
        force_methods = {
            'foreignkey_storage_operator': ('CDNX_get_fk_info_storage_operator', _('---')),
        }
